# Summary
scan_native_pubkeys.ps1 scans Cerberus Native users & groups and checks for RSA public keys with a weak exponent

# Dependencies
* none

# Usage
Basic help: scan_native_pubkeys.ps1 -?  
Full help: Get-Help scan_native_pubkeys.ps1 -full

# Issue Addressed

In OpenSSL 3, FIPS public key verification follows FIPS 186-4 rules; see [section B.3.1](https://nvlpubs.nist.gov/nistpubs/FIPS/NIST.FIPS.186-4.pdf) which requires:

> In addition, all IFC keys shall meet the following criteria in order to conform to FIPS 186-4: 
> 1. The public exponent e shall be selected with the following constraints:  
>    (a.) The public verification exponent e shall be selected prior to generating the primes p and q, and the private signature exponent d.  
>    (b.) The exponent e shall be an odd positive integer such that: 2^16^ < e < 2^256^.
>
> Note that the value of e may be any value that meets constraint 1(b), i.e., e may be either a fixed value or a random value.

So _e_ must be greater than 65536 (bit count greater than 16 and less than 257). However, PuttyGen before version 0.75 uses _e=37_ (bitcount of 6). Such keys will fail validation as they are not FIPS compliant.

Beginning with version 0.75, PuttyGen changed their _e_ value to 65537 and new keys will be FIPS compliant.

This script will scan your native Cerberus users & groups and determine if any of the keys are in a format generated by PuttyGen and test them.

If a public key fails, a message showing the user/group affected and the full path to the public key will be displayed.

# Notes
If you have issues with security policy, use the bypass:
_Example:_  
```powershell.exe -executionpolicy bypass -file .\scan_native_pubkeys.ps1```
